searchData={"items":[{"type":"module","title":"Peri","doc":"Peri is a schema validation library for Elixir, inspired by Clojure's Plumatic Schema.\nIt provides a flexible and powerful way to define and validate data structures using schemas.\nThe library supports nested schemas, optional fields, custom validation functions, and various type constraints.","ref":"Peri.html"},{"type":"module","title":"Key Features - Peri","doc":"- **Simple and Nested Schemas**: Define schemas that can handle complex, nested data structures.\n- **Optional and Required Fields**: Specify fields as optional or required with type constraints.\n- **Custom Validation Functions**: Use custom functions to validate fields.\n- **Comprehensive Error Handling**: Provides detailed error messages for validation failures.\n- **Type Constraints**: Supports various types including enums, lists, maps, tuples, literals, and more.","ref":"Peri.html#module-key-features"},{"type":"module","title":"Usage - Peri","doc":"To define a schema, use the `defschema` macro. By default, all fields in the schema are optional unless specified otherwise.\n\n```elixir\ndefmodule MySchemas do\n  import Peri\n\n  defschema :user, %{\n    name: :string,\n    age: :integer,\n    email: {:required, :string},\n    address: %{\n      street: :string,\n      city: :string\n    },\n    tags: {:list, :string},\n    role: {:enum, [:admin, :user, :guest]},\n    geolocation: {:tuple, [:float, :float]},\n    preferences: {:map, :string},\n    scores: {:map, :string, :integer},\n    status: {:literal, :active},\n    rating: {:custom, &validate_rating/1}\n  }\n\n  defp validate_rating(n) when n < 10, do: :ok\n  defp validate_rating(_), do: {:error, \"invalid rating\", []}\nend\n```\n\nYou can then use the schema to validate data:\n\n```elixir\nuser_data = %{\n  name: \"John\", age: 30, email: \"john@example.com\",\n  address: %{street: \"123 Main St\", city: \"Somewhere\"},\n  tags: [\"science\", \"funky\"], role: :admin,\n  geolocation: {12.2, 34.2}, \n  preferences: %{\"theme\" => \"dark\", \"notifications\" => \"enabled\"},\n  scores: %{\"math\" => 95, \"science\" => 92},\n  status: :active,\n  rating: 9\n}\n\ncase MySchemas.user(user_data) do\n  {:ok, valid_data} -> IO.puts(\"Data is valid!\")\n  {:error, errors} -> IO.inspect(errors, label: \"Validation errors\")\nend\n```","ref":"Peri.html#module-usage"},{"type":"module","title":"Error Handling - Peri","doc":"Peri provides detailed error messages that include the path to the invalid data, the expected and actual values, and custom error messages for custom validations.","ref":"Peri.html#module-error-handling"},{"type":"module","title":"Schema Types - Peri","doc":"Peri supports the following schema types:\n\n- `:string`, `:integer`, `:float`, `:boolean`, `:atom`, `:map`, `:pid` - Basic types\n- `{:required, type}` - Mark a field as required\n- `{:list, type}` - List of elements of the given type\n- `{:map, type}` - Map with values of the given type\n- `{:map, key_type, value_type}` - Map with keys and values of specified types\n- `{:tuple, [type1, type2, ...]}` - Tuple with elements of specified types\n- `{:enum, [value1, value2, ...]}` - One of the specified values\n- `{:literal, value}` - Exactly matches the specified value\n- `{:either, {type1, type2}}` - Either type1 or type2\n- `{:oneof, [type1, type2, ...]}` - One of the specified types\n- Nested maps for complex structures","ref":"Peri.html#module-schema-types"},{"type":"module","title":"Functions - Peri","doc":"- `validate/2` - Validates data against a schema.\n- `conforms?/2` - Checks if data conforms to a schema.\n- `validate_schema/1` - Validates the schema definition.\n- `generate/1` - Generates sample data based on schema (when StreamData is available).","ref":"Peri.html#module-functions"},{"type":"module","title":"Example - Peri","doc":"```elixir\ndefmodule MySchemas do\n  import Peri\n\n  defschema :user, %{\n    name: :string,\n    age: :integer,\n    email: {:required, :string}\n  }\nend\n\nuser_data = %{name: \"John\", age: 30, email: \"john@example.com\"}\ncase MySchemas.user(user_data) do\n  {:ok, valid_data} -> IO.puts(\"Data is valid!\")\n  {:error, errors} -> IO.inspect(errors, label: \"Validation errors\")\nend\n```","ref":"Peri.html#module-example"},{"type":"type","title":"Peri.cond_def/0","doc":"","ref":"Peri.html#t:cond_def/0"},{"type":"function","title":"Peri.conforms?/2","doc":"Checks if the given data conforms to the specified schema.","ref":"Peri.html#conforms?/2"},{"type":"function","title":"Parameters - Peri.conforms?/2","doc":"- `schema`: The schema definition to validate against.\n  - `data`: The data to be validated.","ref":"Peri.html#conforms?/2-parameters"},{"type":"function","title":"Returns - Peri.conforms?/2","doc":"- `true` if the data conforms to the schema.\n  - `false` if the data does not conform to the schema.","ref":"Peri.html#conforms?/2-returns"},{"type":"function","title":"Examples - Peri.conforms?/2","doc":"iex> schema = %{name: :string, age: :integer}\n    iex> data = %{name: \"Alice\", age: 30}\n    iex> Peri.conforms?(schema, data)\n    true\n\n    iex> invalid_data = %{name: \"Alice\", age: \"thirty\"}\n    iex> Peri.conforms?(schema, invalid_data)\n    false","ref":"Peri.html#conforms?/2-examples"},{"type":"type","title":"Peri.custom_def/0","doc":"","ref":"Peri.html#t:custom_def/0"},{"type":"type","title":"Peri.default_def/0","doc":"","ref":"Peri.html#t:default_def/0"},{"type":"macro","title":"Peri.defschema/2","doc":"Defines a schema with a given name and schema definition.","ref":"Peri.html#defschema/2"},{"type":"macro","title":"Examples - Peri.defschema/2","doc":"defmodule MySchemas do\n      import Peri\n\n      defschema :user, %{\n        name: :string,\n        age: :integer,\n        email: {:required, :string}\n      }\n    end\n\n    user_data = %{name: \"John\", age: 30, email: \"john@example.com\"}\n    MySchemas.user(user_data)\n    # => {:ok, %{name: \"John\", age: 30, email: \"john@example.com\"}}\n\n    invalid_data = %{name: \"John\", age: 30}\n    MySchemas.user(invalid_data)\n    # => {:error, [email: \"is required\"]}","ref":"Peri.html#defschema/2-examples"},{"type":"type","title":"Peri.dependent_def/0","doc":"","ref":"Peri.html#t:dependent_def/0"},{"type":"type","title":"Peri.float_def/0","doc":"","ref":"Peri.html#t:float_def/0"},{"type":"function","title":"Peri.generate/1","doc":"Generates sample data based on the given schema definition using `StreamData`.\n\nThis function validates the schema first, and if the schema is valid, it uses the\n`Peri.Generatable.gen/1` function to generate data according to the schema.\n\nNote that this function returns a `Stream`, so you traverse easily the data generations.","ref":"Peri.html#generate/1"},{"type":"function","title":"Parameters - Peri.generate/1","doc":"- `schema`: The schema definition to generate data for.","ref":"Peri.html#generate/1-parameters"},{"type":"function","title":"Returns - Peri.generate/1","doc":"- `{:ok, stream}` if the data is successfully generated.\n  - `{:error, errors}` if there are validation errors in the schema.","ref":"Peri.html#generate/1-returns"},{"type":"function","title":"Examples - Peri.generate/1","doc":"iex> schema = %{name: :string, age: {:integer, {:range, {18, 65}}}}\n    iex> {:ok, stream} = Peri.generate(schema)\n    iex> [data] = Enum.take(stream, 1)\n    iex> is_map(data)\n    true\n    iex> data[:age] in 18..65\n    true","ref":"Peri.html#generate/1-examples"},{"type":"function","title":"Peri.get_enumerable_value/2","doc":"","ref":"Peri.html#get_enumerable_value/2"},{"type":"type","title":"Peri.int_def/0","doc":"","ref":"Peri.html#t:int_def/0"},{"type":"macro","title":"Peri.is_enumerable/1","doc":"Checks if the given data is an enumerable, specifically a map or a list.","ref":"Peri.html#is_enumerable/1"},{"type":"macro","title":"Parameters - Peri.is_enumerable/1","doc":"- `data`: The data to check.","ref":"Peri.html#is_enumerable/1-parameters"},{"type":"macro","title":"Examples - Peri.is_enumerable/1","doc":"iex> is_enumerable(%{})\n    true\n\n    iex> is_enumerable([])\n    true\n\n    iex> is_enumerable(123)\n    false\n\n    iex> is_enumerable(\"string\")\n    false","ref":"Peri.html#is_enumerable/1-examples"},{"type":"macro","title":"Peri.is_numeric/1","doc":"Checks if the given data is a numeric value, specifically a integer or a float.","ref":"Peri.html#is_numeric/1"},{"type":"macro","title":"Parameters - Peri.is_numeric/1","doc":"- `data`: The data to check.","ref":"Peri.html#is_numeric/1-parameters"},{"type":"macro","title":"Examples - Peri.is_numeric/1","doc":"iex> is_numeric(123)\n    true\n\n    iex> is_numeric(0xFF)\n    true\n\n    iex> is_numeric(12.12)\n    true\n\n    iex> is_numeric(\"string\")\n    false\n\n    iex> is_numeric(%{})\n    false","ref":"Peri.html#is_numeric/1-examples"},{"type":"macro","title":"Peri.is_numeric_type/1","doc":"Checks if the given type as an atom is a numeric (integer or float).","ref":"Peri.html#is_numeric_type/1"},{"type":"macro","title":"Parameters - Peri.is_numeric_type/1","doc":"- `data`: The data to check.","ref":"Peri.html#is_numeric_type/1-parameters"},{"type":"macro","title":"Examples - Peri.is_numeric_type/1","doc":"iex> is_numeric(:integer)\n    true\n\n    iex> is_numeric(:float)\n    true\n\n    iex> is_numeric(:list)\n    false\n\n    iex> is_numeric({:enum, _})\n    false","ref":"Peri.html#is_numeric_type/1-examples"},{"type":"type","title":"Peri.literal/0","doc":"","ref":"Peri.html#t:literal/0"},{"type":"function","title":"Peri.put_in_enum/3","doc":"Helper function to put a value into an enum, handling\nnot only maps and keyword lists but also structs.","ref":"Peri.html#put_in_enum/3"},{"type":"function","title":"Examples - Peri.put_in_enum/3","doc":"iex> Peri.put_in_enum(%{}, :hello, \"world\")\n    iex> Peri.put_in_enum(%{}, \"hello\", \"world\")\n    iex> Peri.put_in_enum(%User{}, :hello, \"world\")\n    iex> Peri.put_in_enum([], :hello, \"world\")","ref":"Peri.html#put_in_enum/3-examples"},{"type":"type","title":"Peri.schema/0","doc":"","ref":"Peri.html#t:schema/0"},{"type":"type","title":"Peri.schema_def/0","doc":"","ref":"Peri.html#t:schema_def/0"},{"type":"type","title":"Peri.string_def/0","doc":"","ref":"Peri.html#t:string_def/0"},{"type":"type","title":"Peri.time_def/0","doc":"","ref":"Peri.html#t:time_def/0"},{"type":"function","title":"Peri.to_changeset!/2","doc":"Converts a `Peri.schema()` definition to an Ecto [schemaless changesets](https://hexdocs.pm/ecto/Ecto.Changeset.html#module-schemaless-changesets).","ref":"Peri.html#to_changeset!/2"},{"type":"type","title":"Peri.transform_def/0","doc":"","ref":"Peri.html#t:transform_def/0"},{"type":"function","title":"Peri.validate/2","doc":"Validates a given data map against a schema.\n\nReturns `{:ok, data}` if the data is valid according to the schema, or `{:error, errors}` if there are validation errors.","ref":"Peri.html#validate/2"},{"type":"function","title":"Parameters - Peri.validate/2","doc":"- schema: The schema definition map.\n  - data: The data map to be validated.","ref":"Peri.html#validate/2-parameters"},{"type":"function","title":"Examples - Peri.validate/2","doc":"schema = %{\n      name: :string,\n      age: :integer,\n      email: {:required, :string}\n    }\n\n    data = %{name: \"John\", age: 30, email: \"john@example.com\"}\n    Peri.validate(schema, data)\n    # => {:ok, %{name: \"John\", age: 30, email: \"john@example.com\"}}\n\n    invalid_data = %{name: \"John\", age: 30}\n    Peri.validate(schema, invalid_data)\n    # => {:error, [email: \"is required\"]}","ref":"Peri.html#validate/2-examples"},{"type":"function","title":"Peri.validate_schema/1","doc":"Validates a schema definition to ensure it adheres to the expected structure and types.\n\nThis function can handle both simple and complex schema definitions, including nested schemas, custom validation functions, and various type constraints.","ref":"Peri.html#validate_schema/1"},{"type":"function","title":"Parameters - Peri.validate_schema/1","doc":"- `schema` - The schema definition to be validated. It can be a map or a keyword list representing the schema.","ref":"Peri.html#validate_schema/1-parameters"},{"type":"function","title":"Returns - Peri.validate_schema/1","doc":"- `{:ok, schema}` - If the schema is valid, returns the original schema.\n  - `{:error, errors}` - If the schema is invalid, returns an error tuple with detailed error information.","ref":"Peri.html#validate_schema/1-returns"},{"type":"function","title":"Examples - Peri.validate_schema/1","doc":"Validating a simple schema:\n\n  ```elixir\n  schema = %{\n    name: :string,\n    age: :integer,\n    email: {:required, :string}\n  }\n  assert {:ok, ^schema} = validate_schema(schema)\n  ```\n\n  Validating a nested schema:\n\n  ```elixir\n  schema = %{\n    user: %{\n      name: :string,\n      profile: %{\n        age: {:required, :integer},\n        email: {:required, :string}\n      }\n    }\n  }\n  assert {:ok, ^schema} = validate_schema(schema)\n  ```\n\n  Handling invalid schema definition:\n\n  ```elixir\n  schema = %{\n    name: :str,\n    age: :integer,\n    email: {:required, :string}\n  }\n  assert {:error, _errors} = validate_schema(schema)\n  ```","ref":"Peri.html#validate_schema/1-examples"},{"type":"type","title":"Peri.validation/0","doc":"","ref":"Peri.html#t:validation/0"},{"type":"module","title":"Peri.Generatable","doc":"A module for generating sample data based on Peri schemas using StreamData.\n\nThis module provides functions to generate various types of data, conforming to the schema definitions given in Peri. It leverages the StreamData library to create streams of random data that match the specified types and constraints.","ref":"Peri.Generatable.html"},{"type":"module","title":"Examples - Peri.Generatable","doc":"iex> schema = %{\n    ...>   name: :string,\n    ...>   age: {:integer, {:gte, 18}},\n    ...>   active: :boolean\n    ...> }\n    iex> Peri.Generatable.gen(schema)\n    %StreamData{\n      type: :fixed_map,\n      data: %{name: StreamData.string(:alphanumeric), age: StreamData.filter(StreamData.integer(), &(&1 >= 18)), active: StreamData.boolean()}\n    }","ref":"Peri.Generatable.html#module-examples"},{"type":"function","title":"Peri.Generatable.gen/1","doc":"Generates a stream of data based on the given schema type.\n\nThis function provides various clauses to handle different types and constraints defined in Peri schemas. It uses StreamData to generate streams of random data conforming to the specified types and constraints.","ref":"Peri.Generatable.html#gen/1"},{"type":"function","title":"Parameters - Peri.Generatable.gen/1","doc":"- `schema`: The schema type to generate data for. It can be a simple type like `:integer`, `:string`, etc., or a complex type with constraints like `{:integer, {:gte, 18}}`.","ref":"Peri.Generatable.html#gen/1-parameters"},{"type":"function","title":"Returns - Peri.Generatable.gen/1","doc":"- A StreamData generator stream for the specified schema type.","ref":"Peri.Generatable.html#gen/1-returns"},{"type":"function","title":"Examples - Peri.Generatable.gen/1","doc":"iex> Peri.Generatable.gen(:atom)\n    %StreamData{type: :atom, data: ...}\n\n    iex> Peri.Generatable.gen(:string)\n    %StreamData{type: :string, data: ...}\n\n    iex> Peri.Generatable.gen(:integer)\n    %StreamData{type: :integer, data: ...}\n\n    iex> Peri.Generatable.gen({:enum, [:admin, :user, :guest]})\n    %StreamData{type: :one_of, data: ...}\n\n    iex> Peri.Generatable.gen({:list, :integer})\n    %StreamData{type: :list_of, data: ...}\n\n    iex> Peri.Generatable.gen({:tuple, [:string, :integer]})\n    %StreamData{type: :tuple, data: ...}\n\n    iex> Peri.Generatable.gen({:integer, {:gt, 10}})\n    %StreamData{type: :filter, data: ...}\n\n    iex> Peri.Generatable.gen({:string, {:regex, ~r/^[a-z]+$/}})\n    %StreamData{type: :filter, data: ...}\n\n    iex> Peri.Generatable.gen({:either, {:integer, :string}})\n    %StreamData{type: :one_of, data: ...}\n\n    iex> Peri.Generatable.gen({:custom, {MyModule, :my_fun}})\n    %StreamData{type: :filter, data: ...}\n\n    iex> schema = %{name: :string, age: {:integer, {:gte, 18}}}\n    iex> Peri.Generatable.gen(schema)\n    %StreamData{type: :fixed_map, data: ...}","ref":"Peri.Generatable.html#gen/1-examples"},{"type":"module","title":"Peri.Parser","doc":"The `Peri.Parser` module is responsible for managing the state during schema validation. \nIt centralizes functions to handle updating data, adding errors, and managing the path within \nthe data structure being validated.","ref":"Peri.Parser.html"},{"type":"module","title":"Struct - Peri.Parser","doc":"The `Peri.Parser` struct has the following fields:\n\n- `:data` - The current state of the data being validated.\n- `:errors` - A list of errors encountered during validation.\n- `:path` - The current path within the data structure being validated.","ref":"Peri.Parser.html#module-struct"},{"type":"function","title":"Peri.Parser.add_error/2","doc":"Adds an error to the parser state's list of errors.","ref":"Peri.Parser.html#add_error/2"},{"type":"function","title":"Parameters - Peri.Parser.add_error/2","doc":"- `state` - The current `Peri.Parser` state.\n- `err` - The `%Peri.Error{}` struct representing the error to add.","ref":"Peri.Parser.html#add_error/2-parameters"},{"type":"function","title":"Examples - Peri.Parser.add_error/2","doc":"iex> state = Peri.Parser.new(%{name: \"Alice\"})\n    iex> error = %Peri.Error{path: [:name], message: \"is required\", content: []}\n    iex> Peri.Parser.add_error(state, error)\n    %Peri.Parser{data: %{name: \"Alice\"}, errors: [%Peri.Error{path: [:name], message: \"is required\", content: []}], path: []}","ref":"Peri.Parser.html#add_error/2-examples"},{"type":"function","title":"Peri.Parser.new/2","doc":"Initializes a new `Peri.Parser` struct with the given data.","ref":"Peri.Parser.html#new/2"},{"type":"function","title":"Parameters - Peri.Parser.new/2","doc":"- `data` - The initial data to be validated.","ref":"Peri.Parser.html#new/2-parameters"},{"type":"function","title":"Examples - Peri.Parser.new/2","doc":"iex> Peri.Parser.new(%{name: \"Alice\"})\n    %Peri.Parser{data: %{name: \"Alice\"}, errors: [], path: []}","ref":"Peri.Parser.html#new/2-examples"},{"type":"function","title":"Peri.Parser.update_data/3","doc":"Updates the data in the parser state at the given key with the specified value.","ref":"Peri.Parser.html#update_data/3"},{"type":"function","title":"Parameters - Peri.Parser.update_data/3","doc":"- `state` - The current `Peri.Parser` state.\n- `key` - The key to update in the data.\n- `val` - The value to set at the specified key.","ref":"Peri.Parser.html#update_data/3-parameters"},{"type":"function","title":"Examples - Peri.Parser.update_data/3","doc":"iex> state = Peri.Parser.new(%{name: \"Alice\"})\n    iex> Peri.Parser.update_data(state, :age, 30)\n    %Peri.Parser{data: %{name: \"Alice\", age: 30}, errors: [], path: []}","ref":"Peri.Parser.html#update_data/3-examples"},{"type":"exception","title":"Peri.Error","doc":"Defines the structure and functions for handling validation errors in the Peri schema validation library.\n\nThe `Peri.Error` module encapsulates information about validation errors that occur during schema validation. Each error contains details about the path to the invalid data, the type of error, and any nested errors for complex or deeply nested schemas.","ref":"Peri.Error.html"},{"type":"exception","title":"Attributes - Peri.Error","doc":"- `:message` - A human-readable message describing the error.\n- `:content` - Additional information about the error, such as expected and actual values.\n- `:path` - A list representing the path to the invalid data within the structure being validated.\n- `:key` - The specific key or field that caused the error.\n- `:errors` - A list of nested `Peri.Error` structs for detailed information about nested validation errors.","ref":"Peri.Error.html#module-attributes"},{"type":"exception","title":"Example - Peri.Error","doc":"iex> error = %Peri.Error{\n    ...>   message: \"Validation failed\",\n    ...>   content: %{expected: :string, actual: :integer},\n    ...>   path: [:user, :age],\n    ...>   key: :age,\n    ...>   errors: [\n    ...>     %Peri.Error{\n    ...>       message: \"Expected type string, got integer\",\n    ...>       content: nil,\n    ...>       path: [:user, :age],\n    ...>       key: :age,\n    ...>       errors: nil\n    ...>     }\n    ...>   ]\n    ...> }\n    %Peri.Error{\n      message: \"Validation failed\",\n      content: %{expected: :string, actual: :integer},\n      path: [:user, :age],\n      key: :age,\n      errors: [\n        %Peri.Error{\n          message: \"Expected type string, got integer\",\n          content: nil,\n          path: [:user, :age],\n          key: :age,\n          errors: nil\n        }\n      ]\n    }","ref":"Peri.Error.html#module-example"},{"type":"exception","title":"Functions - Peri.Error","doc":"- `error_to_map/1` - Converts a `Peri.Error` struct to a map, including nested errors.","ref":"Peri.Error.html#module-functions"},{"type":"function","title":"Peri.Error.error_to_json/1","doc":"","ref":"Peri.Error.html#error_to_json/1"},{"type":"function","title":"Peri.Error.error_to_map/1","doc":"Recursively converts a `Peri.Error` struct into a map.","ref":"Peri.Error.html#error_to_map/1"},{"type":"function","title":"Parameters - Peri.Error.error_to_map/1","doc":"- `error` - A `Peri.Error` struct to be transformed.","ref":"Peri.Error.html#error_to_map/1-parameters"},{"type":"function","title":"Examples - Peri.Error.error_to_map/1","doc":"iex> error = %Peri.Error{\n    ...>   message: \"Validation failed\",\n    ...>   content: %{expected: :string, actual: :integer},\n    ...>   path: [:user, :age],\n    ...>   key: :age,\n    ...>   errors: [\n    ...>     %Peri.Error{\n    ...>       message: \"Expected type string, got integer\",\n    ...>       content: nil,\n    ...>       path: [:user, :age],\n    ...>       key: :age,\n    ...>       errors: nil\n    ...>     }\n    ...>   ]\n    ...> }\n    iex> Peri.Error.error_to_map(error)\n    %{\n      message: \"Validation failed\",\n      content: %{expected: :string, actual: :integer},\n      path: [:user, :age],\n      key: :age,\n      errors: [\n        %{\n          message: \"Expected type string, got integer\",\n          content: nil,\n          path: [:user, :age],\n          key: :age,\n          errors: nil\n        }\n      ]\n    }","ref":"Peri.Error.html#error_to_map/1-examples"},{"type":"function","title":"Peri.Error.format_error_message/2","doc":"","ref":"Peri.Error.html#format_error_message/2"},{"type":"function","title":"Peri.Error.new_child/4","doc":"Creates a new child error with a path, key, message, and context.","ref":"Peri.Error.html#new_child/4"},{"type":"function","title":"Parameters - Peri.Error.new_child/4","doc":"- `path` - A list representing the path to the invalid data.\n  - `key` - The specific key or field that caused the error.\n  - `message` - A string template for the error message.\n  - `context` - A list of key-value pairs to replace in the message template.","ref":"Peri.Error.html#new_child/4-parameters"},{"type":"function","title":"Examples - Peri.Error.new_child/4","doc":"iex> Peri.Error.new_child([:user], :age, \"Invalid value for %{field}\", [field: \"age\"])\n    %Peri.Error{\n      path: [:user, :age],\n      key: :age,\n      message: \"Invalid value for age\",\n      content: %{field: \"age\"}\n    }","ref":"Peri.Error.html#new_child/4-examples"},{"type":"function","title":"Peri.Error.new_parent/3","doc":"Creates a new parent error with nested errors.","ref":"Peri.Error.html#new_parent/3"},{"type":"function","title":"Parameters - Peri.Error.new_parent/3","doc":"- `path` - A list representing the path to the invalid data.\n  - `key` - The specific key or field that caused the error.\n  - `errors` - A list of nested `Peri.Error` structs.","ref":"Peri.Error.html#new_parent/3-parameters"},{"type":"function","title":"Examples - Peri.Error.new_parent/3","doc":"iex> Peri.Error.new_parent([:user], :age, [%Peri.Error{message: \"Invalid age\"}])\n    %Peri.Error{\n      path: [:user, :age],\n      key: :age,\n      errors: [%Peri.Error{message: \"Invalid age\"}]\n    }","ref":"Peri.Error.html#new_parent/3-examples"},{"type":"function","title":"Peri.Error.new_single/2","doc":"Creates a new single error with a formatted message and context.","ref":"Peri.Error.html#new_single/2"},{"type":"function","title":"Parameters - Peri.Error.new_single/2","doc":"- `message` - A string template for the error message.\n  - `context` - A list of key-value pairs to replace in the message template.","ref":"Peri.Error.html#new_single/2-parameters"},{"type":"function","title":"Examples - Peri.Error.new_single/2","doc":"iex> Peri.Error.new_single(\"Invalid value for %{field}\", [field: \"age\"])\n    %Peri.Error{\n      message: \"Invalid value for age\",\n      content: %{field: \"age\"}\n    }","ref":"Peri.Error.html#new_single/2-examples"},{"type":"type","title":"Peri.Error.t/0","doc":"","ref":"Peri.Error.html#t:t/0"},{"type":"function","title":"Peri.Error.update_error_paths/2","doc":"","ref":"Peri.Error.html#update_error_paths/2"},{"type":"exception","title":"Peri.InvalidSchema","doc":"Exception raised when an invalid schema is encountered.\n\nThis exception is raised with a list of `Peri.Error` structs,\nproviding a readable message overview of the validation errors.","ref":"Peri.InvalidSchema.html"},{"type":"type","title":"Peri.InvalidSchema.t/0","doc":"","ref":"Peri.InvalidSchema.html#t:t/0"},{"type":"extras","title":"Peri","doc":"# Peri\n\n> [!NOTE]\n> Peri is implementing an Ecto integration right now, you can test it pointing your mix.exs def to the `feat/ecto-integration` branch\n>\n> Something like `{:peri, github: \"zoedsoupe/peri\", branch: \"feat/ecto-integration\"}`\n>\n> You can follow the development status on https://github.com/zoedsoupe/peri/pull/18\n\nPeri is a schema validation library for Elixir, inspired by Clojure's Plumatic Schema. It provides a powerful and flexible way to define and validate schemas for your data, ensuring data integrity and consistency throughout your application. Peri supports a variety of types and validation rules, and it can generate sample data based on your schemas.","ref":"readme.html"},{"type":"extras","title":"Features - Peri","doc":"- **Schema Definition**: Define schemas using a concise and expressive DSL.\n- **Nested Schemas**: Support for deeply nested and complex schemas.\n- **Custom Validation**: Implement custom validation functions for specific requirements.\n- **Error Handling**: Detailed error messages with path information for easy debugging.\n- **Data Generation**: Generate sample data based on your schemas using `StreamData`.","ref":"readme.html#features"},{"type":"extras","title":"Installation - Peri","doc":"Add this line to your `mix.exs`:\n```elixir\ndefp deps do\n  [\n    {:peri, \"~> 0.3\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Available Types - Peri","doc":"- `:any` - Allows any data type.\n- `:atom` - Validates that the field is an atom.\n- `:string` - Validates that the field is a binary (string).\n  - `{:regex, regex}` - Validates that the string field matches a given `regex`\n  - `{:eq, val}` - Validates that the string field is equal to `val`\n  - `{:min, min}` - Validates that the string field has at least the `min` length\n  - `{:max, max}` - Validates that the string field has at maximum the `max` length\n- `:integer` - Validates that the field is an integer.\n  - `{:eq, val}` - Validates taht the integer field is equal to `val`\n  - `{:neq, val}` - Validates taht the integer field is not equal to `val`\n  - `{:lt, val}` - Validates taht the integer field is lesss than `val`\n  - `{:lte, val}` - Validates taht the integer field is less than or equal to `val`\n  - `{:gt, val}` - Validates taht the integer field is greater than `val`\n  - `{:gte, val}` - Validates taht the integer field is greater than or equal to `val`\n  - `{:range, {min, max}}` - Validates taht the integer field is inside the range of `min` to `max` (inclusive)\n- `:float` - Validates that the field is a float.\n- `:boolean` - Validates that the field is a boolean.\n- `:map` - Validates that the field is a map.\n- `{:map, type}` - Validates that the field is a map where all values conform to the specified type.\n- `{:map, key_type, value_type}` - Validates that the field is a map where keys conform to key_type and values conform to value_type.\n- `{:required, type}` - Marks the field as required and validates it according to the specified type.\n- `{:enum, choices}` - Validates that the field is one of the specified choices.\n- `{:list, type}` - Validates that the field is a list of elements of the specified type.\n- `{:literal, value}` - Validates that the field is exactly equal to the specified literal value.\n- `{:tuple, types}` - Validates that the field is a tuple with elements of the specified types.\n- `{type, {:default, default}}` - Provides a default value if the field is missing or `nil`.\n  - `{type, {:default, &some_fun/0}}` - The default values is retrieved from callinf `some_fun/0` if the field is missing.\n  - `{type, {:default, {mod, fun}}}` - The default values is retrieved from callinf `mod.fun/0` if the field is missing.\n- `{type, {:transform, mapper}}` - Transforms the field value using the specified mapper function. It can be a 1 or 2 arity function: when is a single arity the mapper function will only receive the defined field value, while with 2 arity will receive the current defined field value and the whole data as the second argument.\n  - `{type, {:transform, {mod, fun}}}` - Transforms the field value using the specified `mod.fun/1` function. Notice that `fun` can be a 2 arity so it can receive the whole data being validated, in case on dependent fields transformations.\n  - `{type, {:transform, {mod, fun, args}}}` - Transforms the field value using the specified MFA. Notice that `fun` will be at least a 2 arity one so it can receive the whole data being validated, in case on dependent fields transformations and the maximum arity allowed will be 2 + `length(args)`.\n- `{:either, {type1, type2}}` - Validates that the field is either of the two specified types.\n- `{:oneof, types}` - Validates that the field is one of the specified types.\n- `{:custom, callback}` - Validates that the field passes the custom validation function.\n- `{:custom, {mod, fun}}` - Validates that the field passes the custom validation function.\n- `{:custom, {mod, fun, args}}` - Validates that the field passes the custom validation function.\n- `{:dependent, field, condition, type}` - Validates the field based on the value of another field. Check the [dependent schema examples](#dependent-schemas) section for more info.\n- `{:dependent, condition}` - Validates the field based on the value of multiple data values. Check the [dependent schema examples](#dependent-schemas) section for more info.\n  - `{:dependent, {mod, fun}}` - Validates the field based on the value of multiple data values but executes `mod.fun/1` in runtime.\n- `{:cond, condition, type, else_type}` - Conditional validation based on a condition function. Check the [conditional schema examples](#conditional-schemas) section for more info.","ref":"readme.html#available-types"},{"type":"extras","title":"Defining Schemas - Peri","doc":"","ref":"readme.html#defining-schemas"},{"type":"extras","title":"Using the Macro - Peri","doc":"You can define schemas using the `defschema` macro, which provides a concise syntax for defining and validating schemas.\n\n```elixir\ndefmodule MySchemas do\n  import Peri\n\n  defschema :user, %{\n    name: :string,\n    age: {:integer, {:transform, & &1 * 2}},\n    email: {:required, :string},\n    address: %{\n      street: :string,\n      city: :string\n    },\n    tags: {:list, :string},\n    role: {:enum, [:admin, :user, :guest]},\n    status: {:literal, :active},\n    preferences: {:map, :string},\n    scores: {:map, :string, :integer},\n    geolocation: {:tuple, [:float, :float]},\n    rating: {:custom, &validate_rating/1}\n  }\n\n  defp validate_rating(n) when n < 10, do: :ok\n  defp validate_rating(_), do: {:error, \"invalid rating\", []}\nend\n```","ref":"readme.html#using-the-macro"},{"type":"extras","title":"Without the Macro - Peri","doc":"You can also define schemas directly without using the macro:\n\n```elixir\nschema = %{\n  name: :string,\n  age: {:integer, {:transform, & &1 * 2}},\n  email: {:required, :string}\n}\n\nPeri.validate(schema, %{name: \"John\", age: 30, email: \"john@example.com\"})\n```","ref":"readme.html#without-the-macro"},{"type":"extras","title":"Composable and Reusable Schemas - Peri","doc":"Schemas can be composed and reused to build complex data structures.\n\n```elixir\ndefmodule MySchemas do\n  import Peri\n\n  defschema :address, %{\n    street: :string,\n    city: :string\n  }\n\n  defschema :user, %{\n    name: :string,\n    age: :integer,\n    email: {:required, :string},\n    address: get_schema(:address)\n  }\nend\n```","ref":"readme.html#composable-and-reusable-schemas"},{"type":"extras","title":"Conditional Schemas - Peri","doc":"You can define conditional types for a schema based on a callback condition, let's see an example:\n\n```elixir\ndefmodule CondSchema do\n  import Peri\n\n  defschema(:details, %{\n    email: {:required, :string},\n    country: {:required, :string}\n  })\n\n  defschema(:info, %{\n    name: {:required, :string},\n    provide_details: {:required, :boolean},\n    details: {:cond, & &1.provide_details, {:required, get_schema(:details)}, nil}\n  })\nend\n```\n\nIn this example we can read the `info.details` field schema definition as: \"if the `provide_details` field is `true` then the `info.details` field should be parsed as the `details` schema, else, it should be parsed as `nil`\".\n\n> Notice that the condition callback should return boolean","ref":"readme.html#conditional-schemas"},{"type":"extras","title":"Dependent Schemas - Peri","doc":"You can parse fields that depend on onther fields, let's check some examples\n\n#### Single field dependency\n\n```elixir\ndefmodule UserSchemas do\n  import Peri\n\n  defschema :user_registration, %{\n    username: {:required, :string},\n    password: {:required, :string},\n    password_confirmation: {:dependent, :password, &validate_confirmation/2, :string}\n  }\n\n  # if confirmation has the same value of password, the validation is ok\n  defp validate_confirmation(%{password: password}, password), do: :ok\n\n  defp validate_confirmation(_confirmation, _password) do\n    {:error, \"confirmation should be equal to password\", []}\n  end\nend\n```\n\nIn this example we say that \"the `user_registration.password_confirmation` field should be parsed as `string` only if it passes the `validate_confirmation/2` function, which in this case asserts that the `user_registration.password` field should be equal to the confirmation one.\"\n\nThe callback passed to this type definition should be a 2 arity function that will receive the current nest level data as first argument and the value of the current field as the second argument and it should return only `:ok` or `{:error, template, context}`.\n\n#### Multiple fields dependencies and custom parsing\n\nA more complex dependent type schema definition would be:\n\n```elixir\ndefmodule TypeDependentSchema do\n  import Peri\n\n  defschema(:email_details, %{email: {:required, :string}})\n\n  defschema(:country_details, %{country: {:required, :string}})\n\n  defschema(:details, Map.merge(get_schema(:email_details), get_schema(:country_details)))\n\n  defschema(:info, %{\n    name: {:required, :string},\n    provide_email: {:required, :boolean},\n    provide_country: {:required, :boolean},\n    details: {:dependent, &verify_details/1}\n  })\n\n  defp verify_details(%{data: data}) do\n    %{provide_email: pe, provide_country: pc} = data\n\n    provide = {pe, pc}\n\n    case provide do\n      {true, true} -> {:ok, {:required, get_schema(:details)}}\n      {true, false} -> {:ok, {:required, get_schema(:email_details)}}\n      {false, true} -> {:ok, {:required, get_schema(:country_details)}}\n      {false, false} -> {:ok, nil}\n    end\n  end\nend\n```\n\nIn this example we have different schemas parsing rules based on the structure and values of the given data. Basically this type deifinition could be read as:\n\n- if the field `info.provide_email` and `info.provide_country` is both `true`, then the `info.details` field is required to provide both `email` and `country` fields.\n- if the field `info.provide_email` is `true` but `info.provide_country` is `false`, so `info.details` should only contains the `info.details.email` field.\n- if the field `info.provide_email` is `false` but `info.provide_country` is `true`, so `info.details` should only contains the `info.details.country` field.\n- if both `info.provide_email` and `info.provide_country` are `false`, so `info.details` should be parsed as `nil`.\n\nNotice that this kind of dependent type definition should return `{:ok, type}` whereas `type` is a valid Peri schema, or `{:error, template, context}`.","ref":"readme.html#dependent-schemas"},{"type":"extras","title":"Custom Validation Functions - Peri","doc":"Implement custom validation functions to handle specific validation logic.\n\nThe spec of the custom validation function is:\n```elixir\n@spec validation(term) :: :ok | {:error, template :: String.t(), context :: map | keyword}\n```\n\nWhere `template` is a template string with the notation of `%{value}` where `value` is the name of the variable to be injected on the template. And `context` is a map or keyword list where the key is the name of the variable that will be injected into the template and the value is the value of this injected variable. Let's see an example:\n\n```elixir\ndefmodule MySchemas do\n  import Peri\n\n  defschema :user, %{\n    name: :string,\n    age: {:custom, &validate_age/1}\n  }\n\n  defp validate_age(age) when age >= 0 and age <= 120, do: :ok\n  defp validate_age(age), do: {:error, \"invalid age, received: %{age}\", [age: age]}\nend\n```","ref":"readme.html#custom-validation-functions"},{"type":"extras","title":"Error Handling with Peri.Error - Peri","doc":"Peri provides detailed error messages to help identify validation issues. Errors include path information to pinpoint the exact location of the error in the data structure.\n\n```elixir\ncase Peri.validate(schema, data) do\n  {:ok, valid_data} -> IO.puts(\"Data is valid!\")\n  {:error, errors} -> IO.inspect(errors, label: \"Validation errors\")\nend\n```","ref":"readme.html#error-handling-with-peri-error"},{"type":"extras","title":"Data Generation - Peri","doc":"Peri can generate sample data based on your schemas using `StreamData`.\n\nFor this feature to work, ensures that you application depends on [stream_data](https://hexdocs.pm/stream_data).\n\n```elixir\nschema = %{\n  name: :string,\n  age: {:integer, {:gte, 18}},\n  active: :boolean\n}\n\nsample_data = Peri.generate(schema)\nEnum.take(sample_data, 10) # Generates 10 samples of the schema\n```","ref":"readme.html#data-generation"},{"type":"extras","title":"Perfect for Raw Data Structures - Peri","doc":"Peri excels in validating raw data structures, such as tuples, strings, lists, and integers, with extensive validation options. This makes it ideal for use cases where you need to enforce strict data integrity rules on a wide variety of data types. Here's how Peri can help you handle these data structures:","ref":"readme.html#perfect-for-raw-data-structures"},{"type":"extras","title":"Tuples - Peri","doc":"Tuples can be validated for their structure and content, ensuring each element meets specific criteria.\n\n```elixir\ndefmodule MySchemas do\n  import Peri\n\n  defschema :coordinates, {:tuple, [:float, :float]}\nend\n\ndata = {12.34, 56.78}\nPeri.validate(get_schema(:coordinates), data)\n# => {:ok, {12.34, 56.78}}\n\ninvalid_data = {12.34, \"not a float\"}\nPeri.validate(get_schema(:coordinates), invalid_data)\n# => {:error, [%Peri.Error{message: \"expected type of :float received \\\"not a float\\\" value\"}]}\n```","ref":"readme.html#tuples"},{"type":"extras","title":"Strings - Peri","doc":"Strings can be validated for length, equality, and matching regular expressions.\n\n```elixir\ndefmodule MySchemas do\n  import Peri\n\n  defschema :username, {:string, {:regex, ~r/^[a-zA-Z0-9_]+$/}}\nend\n\nvalid_data = %{username: \"valid_user\"}\nPeri.validate(get_schema(:username), valid_data)\n# => {:ok, %{username: \"valid_user\"}}\n\ninvalid_data = %{username: \"invalid user\"}\nPeri.validate(get_schema(:username), invalid_data)\n# => {:error, [%Peri.Error{message: \"should match the ~r/^[a-zA-Z0-9_]+$/ pattern\"}]}\n```","ref":"readme.html#strings"},{"type":"extras","title":"Lists - Peri","doc":"Lists can be validated to ensure all elements are of a specific type and meet certain criteria.\n\n```elixir\ndefmodule MySchemas do\n  import Peri\n\n  defschema :tags, {:list, :string}\nend\n\nvalid_data = %{tags: [\"elixir\", \"programming\"]}\nPeri.validate(get_schema(:tags), valid_data)\n# => {:ok, %{tags: [\"elixir\", \"programming\"]}}\n\ninvalid_data = %{tags: [\"elixir\", 42]}\nPeri.validate(get_schema(:tags), invalid_data)\n# => {:error, [%Peri.Error{message: \"expected type of :string received 42 value\"}]}\n```","ref":"readme.html#lists"},{"type":"extras","title":"Integers - Peri","doc":"Integers can be validated for equality, inequality, and range constraints.\n\n```elixir\ndefmodule MySchemas do\n  import Peri\n\n  defschema :age, {:integer, {:range, {18, 65}}}\nend\n\nvalid_data = %{age: 30}\nPeri.validate(get_schema(:age), valid_data)\n# => {:ok, %{age: 30}}\n\ninvalid_data = %{age: 17}\nPeri.validate(get_schema(:age), invalid_data)\n# => {:error, [%Peri.Error{message: \"should be in the range of 18..65 (inclusive)\"}]}\n```","ref":"readme.html#integers"},{"type":"extras","title":"Maps with Type Constraints - Peri","doc":"Maps can be validated to ensure all values match a specific type or that both keys and values match specified types.\n\n```elixir\ndefmodule MySchemas do\n  import Peri\n\n  defschema :user_preferences, {:map, :string}\n  defschema :user_scores, {:map, :string, :integer}\nend\n\n# Map with string values\nvalid_prefs = %{theme: \"dark\", notifications: \"enabled\"}\nPeri.validate(get_schema(:user_preferences), valid_prefs)\n# => {:ok, %{theme: \"dark\", notifications: \"enabled\"}}\n\n# Map with string keys and integer values\nvalid_scores = %{\"math\" => 95, \"science\" => 92}\nPeri.validate(get_schema(:user_scores), valid_scores)\n# => {:ok, %{\"math\" => 95, \"science\" => 92}}\n\n# Invalid data with wrong value type\ninvalid_prefs = %{theme: \"dark\", notifications: true}\nPeri.validate(get_schema(:user_preferences), invalid_prefs)\n# => {:error, [%Peri.Error{message: \"expected type of :string received true value\"}]}\n```","ref":"readme.html#maps-with-type-constraints"},{"type":"extras","title":"Literal Values - Peri","doc":"Literal values can be validated to ensure they exactly match a specific value.\n\n```elixir\ndefmodule MySchemas do\n  import Peri\n\n  defschema :user_status, {:literal, :active}\nend\n\nvalid_data = :active\nPeri.validate(get_schema(:user_status), valid_data)\n# => {:ok, :active}\n\ninvalid_data = :inactive\nPeri.validate(get_schema(:user_status), invalid_data)\n# => {:error, [%Peri.Error{message: \"expected literal value :active but got :inactive\"}]}\n```","ref":"readme.html#literal-values"},{"type":"extras","title":"Comprehensive Validation Options - Peri","doc":"Peri's robust validation capabilities make it suitable for various data types and validation needs:\n\n- **Equality and Inequality**: Validate that values match or do not match specific criteria.\n- **Ranges**: Ensure numerical values fall within specified bounds.\n- **Regular Expressions**: Enforce patterns on string data.\n- **Map Constraints**: Validate maps with type constraints on keys and values.\n- **Literal Values**: Ensure exact matches with specified values. \n- **Custom Validation**: Implement complex rules through custom functions.\n\nBy supporting these raw data structures and providing detailed error handling, Peri ensures that your data remains consistent and adheres to the defined rules, making it an excellent choice for applications requiring strict data validation.","ref":"readme.html#comprehensive-validation-options"},{"type":"extras","title":"Comparison with other data validation and mapping libraries - Peri","doc":"","ref":"readme.html#comparison-with-other-data-validation-and-mapping-libraries"},{"type":"extras","title":"Peri vs. Norm - Peri","doc":"**Norm** is another Elixir library for schema and data validation. While it shares some similarities with Peri, there are distinct differences:\n\n- **Focus**: Norm focuses on defining specifications and generating tests, offering a more comprehensive approach to data specifications, while Peri focuses on schema validation and transformations.\n- **Validation**: Peri provides a more extensive set of built-in validations for strings, numbers, and custom types, whereas Norm allows for more generic and composable specifications.","ref":"readme.html#peri-vs-norm"},{"type":"extras","title":"Peri vs. Drops - Peri","doc":"**Drops** is another Elixir library designed for validating and casting data. Key differences include:\n\n- **Schema Definition**: Drops uses a different syntax and approach for defining schemas. Peri provides a more familiar and flexible schema definition approach, not relying on macros.\n- **Validation Capabilities**: Peri offers more advanced validation features, such as default values, transformations, and custom validations.\n- **Error Handling**: Both libraries provide detailed error handling, but Peri's `Peri.Error` struct offers more context and customization options for error messages.","ref":"readme.html#peri-vs-drops"},{"type":"extras","title":"Peri vs. Ecto Schemaless Changesets - Peri","doc":"**Ecto** is a powerful data mapping and query generator for Elixir, and it offers schemaless changesets for validating data without defining database schemas.\n\n- **Flexibility**: Peri offers more flexibility in defining schemas for various data structures, such as tuples, nested maps, and lists, along with detailed validation options.\n- **Composable Validations**: Peri allows for more composable and reusable schemas, making it ideal for scenarios where data structures are complex and nested.","ref":"readme.html#peri-vs-ecto-schemaless-changesets"},{"type":"extras","title":"Peri vs. Ecto Embedded Changesets - Peri","doc":"Ecto embedded changesets are used for validating and casting nested structures within Ecto schemas.\n\n- **Nested Structures**: Both Peri and Ecto handle nested structures well. However, Peri provides more granular control over validation rules for different types of nested data.\n- **Usage Context**: Ecto embedded changesets are tightly integrated with Ecto schemas and structs, whereas Peri can be used independently of any kind of data structure or type, making it more versatile for various use cases.\n- **Custom Validations**: Peri's custom validation functions and integration with `StreamData` for data generation provide additional capabilities not available in Ecto.","ref":"readme.html#peri-vs-ecto-embedded-changesets"},{"type":"extras","title":"Summary - Peri","doc":"While all these libraries offer data validation capabilities, Peri stands out with its flexibility, comprehensive validation options, and integration with StreamData for data generation. Whether you're dealing with raw data structures, need advanced validation features, or want to generate test data, Peri provides a robust and versatile solution tailored to meet these needs.","ref":"readme.html#summary"},{"type":"extras","title":"Why the Name \"Peri\"? - Peri","doc":"The name \"Peri\" is derived from the Greek word \"περί\" (pronounced \"peri\"), which means \"around\" or \"about.\" This name was chosen to reflect the library's primary purpose: to provide comprehensive and flexible schema validation for data structures in Elixir. Just as \"peri\" suggests encompassing or surrounding something, Peri aims to cover all aspects of data validation, ensuring that data conforms to specified rules and constraints.\n\nThe choice of the name \"Peri\" also hints at the library's ability to handle a wide variety of data types and structures, much like how the term \"around\" can denote versatility and inclusiveness. Whether it's validating nested maps, complex tuples, or strings with specific patterns, Peri is designed to be a robust tool that can adapt to various validation needs in Elixir programming.","ref":"readme.html#why-the-name-peri"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.0"}}